using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Text;
using System.IO;
using System.Reflection;
using System.Web.Profile;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace XmlProviders.Data
{
    /// <summary>
    /// 
    /// </summary>
    [XmlRoot("profiles", Namespace = "http://schemas.swiftus.com/Profile.xsd")]
    [Serializable()]
    public class ProfileCollection : Collection<Profile>
    {
        private object _lockObject = new object();

        /// <summary>
        /// Initializes a new instance of the <see cref="ProfileCollection"/> class.
        /// </summary>
        public ProfileCollection() : base() { }

        internal ProfileCollection(IList<Profile> profiles) : base(profiles) { }

        /// <summary>
        /// Loads the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns></returns>
        public static ProfileCollection Load(string fileName)
        {
            if (File.Exists(fileName))
            {
                ProfileCollection profiles = null;

                Microsoft.Xml.Serialization.GeneratedAssembly.ProfileCollectionSerializer s =
                    new Microsoft.Xml.Serialization.GeneratedAssembly.ProfileCollectionSerializer();

                XmlReaderSettings settings = new XmlReaderSettings();
                settings.Schemas.Add(null, XmlReader.Create(Assembly.GetExecutingAssembly().GetManifestResourceStream(typeof(ProfileCollection), "Profile.xsd")));

                using (XmlReader rdr = XmlReader.Create(fileName, settings))
                {
                    if (s.CanDeserialize(rdr))
                    {
                        profiles = (ProfileCollection)s.Deserialize(rdr);
                    }
                }

                return profiles;
            }
            else
            {
                return new ProfileCollection();
            }
        }

        /// <summary>
        /// Saves the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        public void Save(string fileName)
        {
            Microsoft.Xml.Serialization.GeneratedAssembly.ProfileCollectionSerializer s =
                new Microsoft.Xml.Serialization.GeneratedAssembly.ProfileCollectionSerializer();

            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Encoding = System.Text.Encoding.UTF8;

            XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
            ns.Add(string.Empty, "http://schemas.swiftus.com/Profile.xsd");

            lock (_lockObject)
            {
                using (XmlWriter writer = XmlWriter.Create(fileName, settings))
                {
                    s.Serialize(writer, this, ns);
                }
            }
        }

        /// <summary>
        /// Finds the name of the by user.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="authenticationType">Type of the authentication.</param>
        /// <returns></returns>
        public Collection<Profile> FindByUserName(string userName, System.Web.Profile.ProfileAuthenticationOption authenticationType)
        {
            List<Profile> profiles = new List<Profile>();

            foreach (Profile p in this)
            {
                if (string.Compare(userName, p.UserName, true, CultureInfo.CurrentCulture) == 0) 
                {
                    switch (authenticationType)
                    {
                        case System.Web.Profile.ProfileAuthenticationOption.All:
                            profiles.Add(p);
                            break;
                        case System.Web.Profile.ProfileAuthenticationOption.Anonymous:
                            if (p.IsAnonymous) { profiles.Add(p); }
                            break;
                        case System.Web.Profile.ProfileAuthenticationOption.Authenticated:
                            if (!p.IsAnonymous) { profiles.Add(p); }
                            break;
                    } 
                }
            }

            return new Collection<Profile>(profiles);
        }

        /// <summary>
        /// Creates the profile info collection.
        /// </summary>
        /// <param name="authenticationOption">The authentication option.</param>
        /// <returns></returns>
        public ProfileInfoCollection CreateProfileInfoCollection(ProfileAuthenticationOption authenticationOption)
        {
            ProfileInfoCollection pic = new ProfileInfoCollection();

            foreach (Profile p in this)
            {
                switch (authenticationOption)
                {
                    case ProfileAuthenticationOption.All:
                        pic.Add(p.CreateProfileInfo());
                        break;
                    case ProfileAuthenticationOption.Anonymous:
                        if (p.IsAnonymous) { pic.Add(p.CreateProfileInfo()); }
                        break;
                    case ProfileAuthenticationOption.Authenticated:
                        if (!p.IsAnonymous) { pic.Add(p.CreateProfileInfo()); }
                        break;
                }
            }

            return pic;
        }
    }
}
