using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.IO;
using System.Globalization;
using System.Reflection;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;

namespace XmlProviders.Data
{
    /// <summary>
    /// 
    /// </summary>
    [Serializable()]
    [XmlRoot("roles", Namespace = "http://schemas.swiftus.com/Role.xsd")]
    public class RoleCollection : Collection<Role>
    {
        private object _lockObject = new object();

        /// <summary>
        /// Initializes a new instance of the <see cref="RoleCollection"/> class.
        /// </summary>
        public RoleCollection() : base() { }

        internal RoleCollection(IList<Role> roles) : base(roles) { }

        /// <summary>
        /// Loads the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns></returns>
        public static RoleCollection Load(string fileName)
        {
            if (File.Exists(fileName))
            {
                RoleCollection roles = null;

                Microsoft.Xml.Serialization.GeneratedAssembly.RoleCollectionSerializer s =
                    new Microsoft.Xml.Serialization.GeneratedAssembly.RoleCollectionSerializer();

                XmlReaderSettings settings = new XmlReaderSettings();
                settings.Schemas.Add(null, XmlReader.Create(Assembly.GetExecutingAssembly().GetManifestResourceStream(typeof(RoleCollection), "Role.xsd")));

                using (XmlReader rdr = XmlReader.Create(fileName, settings))
                {
                    if (s.CanDeserialize(rdr))
                    {
                        roles = (RoleCollection)s.Deserialize(rdr);
                    }
                }

                return roles;
            }
            else
            {
                return new RoleCollection();
            }
        }

        /// <summary>
        /// Saves the specified file name.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        public void Save(string fileName)
        {
            Microsoft.Xml.Serialization.GeneratedAssembly.RoleCollectionSerializer s =
                new Microsoft.Xml.Serialization.GeneratedAssembly.RoleCollectionSerializer();

            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Encoding = System.Text.Encoding.UTF8;

            XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
            ns.Add(string.Empty, "http://schemas.swiftus.com/Role.xsd");

            lock (_lockObject)
            {
                using (XmlWriter writer = XmlWriter.Create(fileName, settings))
                {
                    s.Serialize(writer, this, ns);
                }
            }
        }

        /// <summary>
        /// Finds the name of the by.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <returns></returns>
        public Role FindByName(string name)
        {
            foreach(Role r in this)
            {
                if (string.Compare(r.Name, name, true, CultureInfo.CurrentCulture) == 0) { return r; }
            }

            return null;
        }
    }
}
